import { ScalarsMap, EnumValuesMap } from './types';
import { DeclarationBlockConfig } from './utils';
import { NamedTypeNode, ListTypeNode, NonNullTypeNode, DirectiveDefinitionNode, NameNode, InputObjectTypeDefinitionNode, InputValueDefinitionNode, EnumTypeDefinitionNode, ScalarTypeDefinitionNode } from 'graphql';
import { FieldDefinitionNode, UnionTypeDefinitionNode, ObjectTypeDefinitionNode, InterfaceTypeDefinitionNode, EnumValueDefinitionNode } from 'graphql/language/ast';
import { OperationVariablesToObject } from './variables-to-object';
export interface ParsedConfig {
    scalars: ScalarsMap;
    enumValues: EnumValuesMap;
    convert: (str: string) => string;
    typesPrefix: string;
}
export interface RawConfig {
    scalars?: ScalarsMap;
    enumValues?: EnumValuesMap;
    namingConvention?: string;
    typesPrefix?: string;
}
export declare class BaseVisitor<TRawConfig extends RawConfig = RawConfig, TPluginConfig extends ParsedConfig = ParsedConfig> {
    protected _parsedConfig: TPluginConfig;
    protected _argumentsTransformer: OperationVariablesToObject;
    protected _declarationBlockConfig: DeclarationBlockConfig;
    constructor(rawConfig: TRawConfig, additionalConfig: TPluginConfig, defaultScalars?: ScalarsMap);
    setDeclarationBlockConfig(config: DeclarationBlockConfig): void;
    setArgumentsTransformer(argumentsTransfomer: OperationVariablesToObject): void;
    readonly config: TPluginConfig;
    readonly scalars: ScalarsMap;
    convertName(name: any, addPrefix?: boolean): string;
    DirectiveDefinition(node: DirectiveDefinitionNode): string;
    NamedType(node: NamedTypeNode): string;
    ListType(node: ListTypeNode): string;
    NonNullType(node: NonNullTypeNode): string;
    InputObjectTypeDefinition(node: InputObjectTypeDefinitionNode): string;
    InputValueDefinition(node: InputValueDefinitionNode): string;
    Name(node: NameNode): string;
    FieldDefinition(node: FieldDefinitionNode): string;
    UnionTypeDefinition(node: UnionTypeDefinitionNode, key: string | number, parent: any): string;
    ObjectTypeDefinition(node: ObjectTypeDefinitionNode, key: number | string, parent: any): string;
    InterfaceTypeDefinition(node: InterfaceTypeDefinitionNode): string;
    ScalarTypeDefinition(node: ScalarTypeDefinitionNode): string;
    EnumTypeDefinition(node: EnumTypeDefinitionNode): string;
    protected buildEnumValuesBlock(values: ReadonlyArray<EnumValueDefinitionNode>): string;
}
