"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var graphql_codegen_plugin_helpers_1 = require("graphql-codegen-plugin-helpers");
var autoBind = require("auto-bind");
var variables_to_object_1 = require("./variables-to-object");
var scalars_1 = require("./scalars");
var BaseVisitor = /** @class */ (function () {
    function BaseVisitor(rawConfig, additionalConfig, defaultScalars) {
        if (defaultScalars === void 0) { defaultScalars = scalars_1.DEFAULT_SCALARS; }
        this._declarationBlockConfig = {};
        this._parsedConfig = __assign({ scalars: __assign({}, (defaultScalars || scalars_1.DEFAULT_SCALARS), (rawConfig.scalars || {})), enumValues: rawConfig.enumValues || {}, convert: rawConfig.namingConvention ? graphql_codegen_plugin_helpers_1.resolveExternalModuleAndFn(rawConfig.namingConvention) : utils_1.toPascalCase, typesPrefix: rawConfig.typesPrefix || '' }, (additionalConfig || {}));
        autoBind(this);
        this._argumentsTransformer = new variables_to_object_1.OperationVariablesToObject(this.scalars, this.convertName);
    }
    BaseVisitor.prototype.setDeclarationBlockConfig = function (config) {
        this._declarationBlockConfig = config;
    };
    BaseVisitor.prototype.setArgumentsTransformer = function (argumentsTransfomer) {
        this._argumentsTransformer = argumentsTransfomer;
    };
    Object.defineProperty(BaseVisitor.prototype, "config", {
        get: function () {
            return this._parsedConfig;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseVisitor.prototype, "scalars", {
        get: function () {
            return this.config.scalars;
        },
        enumerable: true,
        configurable: true
    });
    BaseVisitor.prototype.convertName = function (name, addPrefix) {
        if (addPrefix === void 0) { addPrefix = true; }
        return (addPrefix ? this.config.typesPrefix : '') + this.config.convert(name);
    };
    BaseVisitor.prototype.DirectiveDefinition = function (node) {
        return '';
    };
    BaseVisitor.prototype.NamedType = function (node) {
        var asString = node.name;
        var type = this.scalars[asString] || this.convertName(asString);
        return type;
    };
    BaseVisitor.prototype.ListType = function (node) {
        var asString = node.type;
        return "Array<" + asString + ">";
    };
    BaseVisitor.prototype.NonNullType = function (node) {
        var asString = node.type;
        return asString;
    };
    BaseVisitor.prototype.InputObjectTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node.name))
            .withBlock(node.fields.join('\n')).string;
    };
    BaseVisitor.prototype.InputValueDefinition = function (node) {
        return utils_1.indent(node.name + ": " + node.type + ",");
    };
    BaseVisitor.prototype.Name = function (node) {
        return node.value;
    };
    BaseVisitor.prototype.FieldDefinition = function (node) {
        var typeString = node.type;
        return utils_1.indent(node.name + ": " + typeString + ",");
    };
    BaseVisitor.prototype.UnionTypeDefinition = function (node, key, parent) {
        var _this = this;
        var originalNode = parent[key];
        var possibleTypes = originalNode.types.map(function (t) { return _this.convertName(t.name.value); }).join(' | ');
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node.name))
            .withContent(possibleTypes).string;
    };
    BaseVisitor.prototype.ObjectTypeDefinition = function (node, key, parent) {
        var _this = this;
        var originalNode = parent[key];
        var interfaces = originalNode.interfaces && node.interfaces.length > 0
            ? originalNode.interfaces.map(function (i) { return _this.convertName(i.name.value); }).join(' & ') + ' & '
            : '';
        var typeDefinition = new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node.name))
            .withContent(interfaces)
            .withBlock(node.fields.join('\n')).string;
        var original = parent[key];
        var fieldsWithArguments = original.fields.filter(function (field) { return field.arguments && field.arguments.length > 0; });
        var fieldsArguments = fieldsWithArguments.map(function (field) {
            var name = original.name.value + _this.convertName(field.name.value, false) + 'Args';
            return new utils_1.DeclarationBlock(_this._declarationBlockConfig)
                .export()
                .asKind('type')
                .withName(_this.convertName(name))
                .withBlock(_this._argumentsTransformer.transform(field.arguments)).string;
        });
        return [typeDefinition].concat(fieldsArguments).filter(function (f) { return f; }).join('\n\n');
    };
    BaseVisitor.prototype.InterfaceTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node.name))
            .withBlock(node.fields.join('\n')).string;
    };
    BaseVisitor.prototype.ScalarTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node.name))
            .withContent(this.config.scalars[node.name] || 'any').string;
    };
    BaseVisitor.prototype.EnumTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('enum')
            .withName(this.convertName(node.name))
            .withBlock(this.buildEnumValuesBlock(node.values)).string;
    };
    BaseVisitor.prototype.buildEnumValuesBlock = function (values) {
        var _this = this;
        return values
            .map(function (enumOption) {
            return utils_1.indent("" + _this.convertName(enumOption.name) + _this._declarationBlockConfig.enumNameValueSeparator + " " + utils_1.wrapWithSingleQuotes(_this.config.enumValues[enumOption.name] || enumOption.name));
        })
            .join(', \n');
    };
    return BaseVisitor;
}());
exports.BaseVisitor = BaseVisitor;
//# sourceMappingURL=base-visitor.js.map